      SUBROUTINE advec2(ib,ie,jb,je,kb,ke)

      IMPLICIT NONE

      INCLUDE 'param.txt'
      INCLUDE 'common.txt'

      INTEGER  ib,ie,jb,je,kb,ke

c********************************************************************
c
c     Advec calculates the advection for all variables over the
c     indicated domain. The indices ib,ie,jb,je,kb and ke ("i,j,k" for
c     the direction and "b" and "e" for begin and end) determine the
c     points for which the advection is calculated.
c
c*********************************************************************


      CALL advu(dudtnm,unew,vnew,wnew,rhonew,ru,rp,dr,dphi,zw,zp,i1
     $     ,j1,k1,ib,ie,jb,je,kb,ke)

      CALL advv(dvdtnm,unew,vnew,wnew,rhonew,ru,rp,dr,dphi,zw,zp,i1
     $     ,j1,k1,ib,ie,jb,je,kb,ke)

      CALL advw(dwdtnm,unew,vnew,wnew,rhonew,ru,rp,dr,dphi,zw,zp
     $     ,i1,j1,k1,ib,ie,jb,je,kb,ke)

      CALL advt(dtdtnm,unew,vnew,wnew,rhonew,tnew,ru,rp,dr,dphi,zw,zp,i1
     $     ,j1,k1,ib,ie,jb,je,kb,ke)

      CALL advy(dydtnm,unew,vnew,wnew,rhonew,ynew,ru,rp,dr,dphi,zw,zp,i1
     $     ,j1,k1,ib,ie,jb,je,kb,ke)

      RETURN
      END


