      SUBROUTINE inlet(ub,vb,wb,yb,tb,rho,time,ru,rp,ijet)

      IMPLICIT NONE

      INCLUDE 'param2.txt'
      INCLUDE 'param.txt'

      REAL ub(0:i1,0:j1,0:k1)
      REAL vb(0:i1,0:j1,0:k1)
      REAL wb(0:i1,0:j1,0:k1)
      REAL yb(0:i1,0:j1,0:k1)
      REAL tb(0:i1,0:j1,0:k1)
      REAL rho(0:i1,0:j1,0:k1)
      REAL ru(0:i1),rp(0:i1)

      REAL time
      INTEGER ijet

      REAL win,u1,u2,v1,v2,w1,w2
      
      PARAMETER (win=2.0)

      REAL kpl
      INTEGER kpln

      REAL upipe(0:pi1,0:pj1,0:pk1)
      REAL vpipe(0:pi1,0:pj1,0:pk1)
      REAL wpipe(0:pi1,0:pj1,0:pk1)
      REAL drpipe
      INTEGER aup(0:i1)

      REAL alpha,beta,preexp,incrho,rhoi

      COMMON /indata/ upipe,vpipe,wpipe,drpipe,aup

      COMMON /heatrl/ alpha,beta,preexp,incrho,rhoi
c----------------------------------------------------------------------

c...inject turbulence at lower jet exit...

      IF(1.EQ.1)THEN

         DO i=0,ijet
            DO j=0,j1

               kpln=pkmax*win*time/plz
               kpln=mod(kpln,pkmax)

               kpl=pkmax*win*time/plz
               kpl=mod(kpl,real(pkmax))

               wb(i,j,0)=(kpl-kpln)*(wpipe(i,j,kpln+1) -
     $              wpipe(i,j,kpln))+wpipe(i,j,kpln) + win

               vb(i,j,0)=(kpl-kpln)*(vpipe(i,j,kpln+1) -
     $              vpipe(i,j,kpln))+vpipe(i,j,kpln)

               ub(i,j,0)=(kpl-kpln)*(upipe(i,j,kpln+1) -
     $              upipe(i,j,kpln))+upipe(i,j,kpln)
               
            END DO
         END DO

      END IF

c...inject turbulence at upper jet exit...

      IF(1.EQ.0)THEN

         DO i=0,ijet
            DO j=0,j1

               kpln=pkmax*win*(time+.1)/plz
               kpln=mod(kpln,pkmax)

               kpl=pkmax*win*(time+.1)/plz
               kpl=mod(kpl,real(pkmax))

               wb(i,j,kmax)=-((kpl-kpln)*(wpipe(i,j,kpln+1) -
     $              wpipe(i,j,kpln))+wpipe(i,j,kpln) + win)

               vb(i,j,k1)=(kpl-kpln)*(vpipe(i,j,kpln+1) -
     $              vpipe(i,j,kpln))+vpipe(i,j,kpln)

               ub(i,j,k1)=(kpl-kpln)*(upipe(i,j,kpln+1) -
     $              upipe(i,j,kpln))+upipe(i,j,kpln)
               
            END DO
         END DO
         
      END IF


c...inject turbulence from pipe DNS...

      IF(1.EQ.0)THEN

         kpln=pkmax*win*time/plz
         kpln=mod(kpln,pkmax)

         kpl=pkmax*win*time/plz
         kpl=mod(kpl,real(pkmax))


         DO i=0,ijet
            DO j=0,j1

               w1=(kpl-kpln)*(wpipe(aup(i),j,kpln+1) - wpipe(aup(i),j
     $              ,kpln))+wpipe(aup(i),j,kpln)

               w2=(kpl-kpln)*(wpipe(aup(i)+1,j,kpln+1) - wpipe(aup(i)+1
     $              ,j,kpln))+wpipe(aup(i)+1,j,kpln)

               wb(i,j,0)=w1+(ru(i)-drpipe*aup(i))*(w2-w1)/drpipe



               v1=(kpl-kpln)*(vpipe(aup(i),j,kpln+1) - vpipe(aup(i),j
     $              ,kpln))+vpipe(aup(i),j,kpln)

               v2=(kpl-kpln)*(vpipe(aup(i)+1,j,kpln+1) - vpipe(aup(i)+1
     $              ,j,kpln))+vpipe(aup(i)+1,j,kpln)

               vb(i,j,0)=v1+(ru(i)-drpipe*aup(i))*(v2-v1)/drpipe



               u1=(kpl-kpln)*(upipe(aup(i),j,kpln+1) - upipe(aup(i),j
     $              ,kpln))+upipe(aup(i),j,kpln)

               u2=(kpl-kpln)*(upipe(aup(i)+1,j,kpln+1) - upipe(aup(i)+1
     $              ,j,kpln))+upipe(aup(i)+1,j,kpln)

               ub(i,j,0)=u1+(ru(i)-drpipe*aup(i))*(u2-u1)/drpipe

               
            END DO
         END DO

      END IF

      RETURN
      END

